package com.lpdp.del.tenunapp.function.detail;

import android.content.ContentValues;
import android.content.Context;
import android.content.Intent;
import android.content.pm.PackageManager;
import android.os.AsyncTask;
import android.os.Bundle;
import android.provider.MediaStore;
import android.support.annotation.NonNull;
import android.support.design.widget.CoordinatorLayout;
import android.support.design.widget.Snackbar;
import android.support.v7.app.AppCompatActivity;
import android.support.v7.widget.LinearLayoutManager;
import android.support.v7.widget.RecyclerView;
import android.support.v7.widget.Toolbar;
import android.view.Gravity;
import android.view.LayoutInflater;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.view.WindowManager;
import android.widget.ArrayAdapter;
import android.widget.Button;
import android.widget.ImageView;
import android.widget.ProgressBar;
import android.widget.Spinner;
import android.widget.TextView;
import android.widget.Toast;

import com.afollestad.materialdialogs.MaterialDialog;
import com.bumptech.glide.Glide;
import com.bumptech.glide.load.engine.DiskCacheStrategy;
import com.lpdp.del.tenunapp.R;
import com.lpdp.del.tenunapp.R2;
import com.lpdp.del.tenunapp.dependency.App;
import com.lpdp.del.tenunapp.dependency.models.Algoritma;
import com.lpdp.del.tenunapp.dependency.models.Generate;
import com.lpdp.del.tenunapp.dependency.models.MotifTenun;
import com.lpdp.del.tenunapp.dependency.models.RequestBodyGenerateImg2;
import com.lpdp.del.tenunapp.dependency.models.ResponseGetGeneratedImage;
import com.lpdp.del.tenunapp.dependency.modules.APIModule;
import com.lpdp.del.tenunapp.dependency.network.TenunNetworkInterface;
import com.lpdp.del.tenunapp.function.kristik.KristikGambarAwalActivity;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;

import javax.inject.Inject;

import butterknife.BindView;
import butterknife.ButterKnife;
import io.realm.Realm;
import io.realm.RealmList;
import io.realm.RealmResults;
import io.realm.Sort;
import retrofit2.Call;
import retrofit2.Callback;
import retrofit2.Response;
import timber.log.Timber;

public class DetailMotifActivity extends AppCompatActivity {
    public static final String KEY_ID_MOTIF = "id_motif_key";
    private static final String KEY_BUNDLE_SAVED_URL = "keyGeneratedURL";
    private static final int REQUEST_WRITE_STORAGE = 112;

    public static String OUTPUT_SIZE = "output_size";
    public static String OUTPUT_SHAPE = "output_shape";
    public static String SUM_OF_MATRIX = "sum_of_matrix";
    public static String COLOR = "color";
    @Inject
    Realm realm;
    @Inject
    TenunNetworkInterface tenunNetworkInterface;

    @BindView(R2.id.toolbarDetailMotif)
    Toolbar toolbar;
    @BindView(R2.id.thumbMotif)
    ImageView thumbMotif;
    @BindView(R2.id.textToolbarDetail)
    TextView textToolbar;
    @BindView(R2.id.coordinatorLayoutDetailMotif)
    CoordinatorLayout coordinatorLayout;
    @BindView(R2.id.dividerView)
    View dividerView;
    @BindView(R2.id.progressBarGenerateResult)
    ProgressBar progressBar;
    @BindView(R2.id.titleResult)
    TextView titleResult;

    List<Object> listParamState = new ArrayList<>();
    MaterialDialog dialog2;
    MotifTenun motif;
    MenuItem generateMenu;
    String generatedUrl = "";
    private Algoritma selectedAlgoritma;
    private HashMap<String, Object> paramValue = new HashMap<>();
    private String generatedImg;
    private String idMotif;
    private String namaMotif;
    private RealmResults<Generate> listGenerate;
    RecylerViewAdapterListGenerate myAdapter;
    LinearLayoutManager mLayoutManager;
    RecyclerView recyclerView;

    public static Intent createIntent(Context context, String id) {
        Intent intent = new Intent(context, DetailMotifActivity.class);
        intent.putExtra(KEY_ID_MOTIF, id);
        return intent;
    }

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_detail_motif);
        App.get(getApplicationContext()).getInjector().inject(this);
        ButterKnife.bind(this);
        Intent returnIntent = getIntent();
        idMotif = returnIntent.getStringExtra(KEY_ID_MOTIF);
        Timber.i("id motif : " + idMotif);
        motif = realm.where(MotifTenun.class)
                .equalTo("id", idMotif)
                .findFirst();
        namaMotif = motif.getNamaMotif();
        String imgUrl = APIModule.ENDPOINT + motif.getImageMotif();

        setSupportActionBar(toolbar);
        textToolbar.setText(R.string.generate);
        getSupportActionBar().setDisplayHomeAsUpEnabled(true);

        recyclerView = (RecyclerView)findViewById(R.id.recyclerviewulos_generate);
        recyclerView.setHasFixedSize(true);
        mLayoutManager = new LinearLayoutManager(this);
        recyclerView.setLayoutManager(mLayoutManager);

        titleResult.setText(R.string.generated_text);
        listGenerate = realm.where(Generate.class).equalTo("idMotif",idMotif).findAll().sort("id", Sort.DESCENDING);
        myAdapter = new RecylerViewAdapterListGenerate(this, listGenerate);
        recyclerView.setAdapter(myAdapter);

        Glide.with(this)
                .load(imgUrl)
                .diskCacheStrategy(DiskCacheStrategy.SOURCE)
                .override(800, 500)
                .centerCrop()
                .fitCenter()
                .crossFade()
                .into(thumbMotif);
        thumbMotif.setOnClickListener(v -> startActivity(KristikGambarAwalActivity.createIntent(this,  idMotif)));
    }

    @Override
    public boolean onCreateOptionsMenu(Menu menu) {
        getMenuInflater().inflate(R.menu.detail_motif_menu, menu);
        return true;
    }

    @Override
    public boolean onOptionsItemSelected(MenuItem item) {
        switch (item.getItemId()) {
            case R.id.action_generate:
                generateMenu = item;
                generateMenu.setEnabled(false);
                createDialog();
                break;
            default:
                break;
        }
        return super.onOptionsItemSelected(item);
    }

    private void createDialog() {
        List<Algoritma> listAlgo = realm.copyFromRealm(realm.where(Algoritma.class).findAll());
        List<String> algoTitle = new ArrayList<>();

        for (Algoritma a : listAlgo) {
            algoTitle.add(a.getNamaAlgoritma());
        }

        View dialogView = LayoutInflater.from(this).inflate(R.layout.layout_spinner_algorithm, coordinatorLayout, false);

        dialog2 = new MaterialDialog.Builder(DetailMotifActivity.this)
                .customView(dialogView, true)
                .build();

        dialog2.getWindow().setBackgroundDrawable(null);
        dialog2.getWindow().getAttributes().windowAnimations = R.style.DialogTransition;
        WindowManager.LayoutParams wlp = dialog2.getWindow().getAttributes();
        wlp.gravity = Gravity.TOP;
        wlp.width = WindowManager.LayoutParams.MATCH_PARENT;
        String[] arrAlgo = new String[]{"img_quilting", "img_nps"};
        ArrayAdapter<String> spinnerAdapter = new ArrayAdapter<>(this, android.R.layout.simple_spinner_dropdown_item, arrAlgo);
        Spinner spinner = (Spinner) dialog2.findViewById(R.id.listAlgo);
        Spinner model = (Spinner) dialog2.findViewById(R.id.model);
        Spinner warna = (Spinner) dialog2.findViewById(R.id.warna);
        spinner.setAdapter(spinnerAdapter);

        Button btnGenerate = (Button) dialog2.findViewById(R.id.btnGenerate);

        dialog2.setOnDismissListener(dialog -> generateMenu.setEnabled(true));
        btnGenerate.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                String namaAlgo = spinner.getSelectedItem().toString();
                String namaModel = model.getSelectedItem().toString();
                String namaWarna = warna.getSelectedItem().toString();
                String sourceFile = namaMotif;
                generateNewMotif(namaAlgo, sourceFile, namaModel, namaWarna, idMotif);

                Toast.makeText(getApplicationContext(), "Input: "+namaAlgo +" " +namaModel +" " +namaMotif +" " + namaWarna, Toast.LENGTH_LONG).show();
                dialog2.dismiss();
            }
        });
        dialog2.show();
    }

    private void generateNewMotif(String algoritma, String sourceFile, String model, String warna, String idMotif) {
        progressBar.setVisibility(View.VISIBLE);
        RequestBodyGenerateImg2 requestBodyGenerateImg2 = new RequestBodyGenerateImg2(algoritma, sourceFile, model, warna, idMotif);

        tenunNetworkInterface.generateNewMotif(APIModule.ACCESS_TOKEN_TEMP, requestBodyGenerateImg2).enqueue(new Callback<ResponseGetGeneratedImage>() {
            @Override
            public void onResponse(Call<ResponseGetGeneratedImage> call, Response<ResponseGetGeneratedImage> response) {
                if (response.isSuccessful()) {
                    generatedImg = response.body().getFilename();
                    List<Generate> generate = response.body().getData();
                    updateListGenerate(generate);
                    progressBar.setVisibility(View.GONE);
                } else {
                    titleResult.setText("Error (" + response.code() + ")");
                    Snackbar snackbar = Snackbar.make(coordinatorLayout, "Error to generate", Snackbar.LENGTH_INDEFINITE);
                    snackbar.setAction("DISMISS", v -> {
                        snackbar.dismiss();
                        titleResult.setText(R.string.generated_text);
                    });
                    snackbar.show();
                    progressBar.setVisibility(View.GONE);

                    generateMenu.setEnabled(true);
                }
            }

            @Override
            public void onFailure(Call<ResponseGetGeneratedImage> call, Throwable t) {
                titleResult.setText(t.getLocalizedMessage());
                Snackbar snackbar = Snackbar.make(coordinatorLayout, "Error to generate", Snackbar.LENGTH_INDEFINITE);
                snackbar.setAction("DISMISS", v -> {
                    snackbar.dismiss();
                    titleResult.setText(R.string.generated_text);
                });
                snackbar.show();
                progressBar.setVisibility(View.GONE);

                generateMenu.setEnabled(true);
            }
        });
    }

    public void updateListGenerate(List<Generate> generate){
        realm.beginTransaction();
        listGenerate.deleteAllFromRealm();
        RealmList<Generate> generates = new RealmList<>();
        generates.addAll(generate);
        realm.insertOrUpdate(generates);
        realm.commitTransaction();
        listGenerate = realm.where(Generate.class).equalTo("idMotif",idMotif).findAll().sort("id", Sort.DESCENDING);
        myAdapter.notifyDataSetChanged();
    }

    @Override
    protected void onSaveInstanceState(Bundle outState) {
        outState.putString(KEY_BUNDLE_SAVED_URL, generatedUrl);
        super.onSaveInstanceState(outState);
    }

    @Override
    protected void onRestoreInstanceState(Bundle savedInstanceState) {
        super.onRestoreInstanceState(savedInstanceState);
    }

    @Override
    public void onRequestPermissionsResult(int requestCode, @NonNull String[] permissions, @NonNull int[] grantResults) {
        super.onRequestPermissionsResult(requestCode, permissions, grantResults);
        switch (requestCode) {
            case REQUEST_WRITE_STORAGE: {
                if (grantResults.length > 0 && grantResults[0] == PackageManager.PERMISSION_GRANTED) {
//                    saveImg();
                } else {
                    Toast.makeText(DetailMotifActivity.this, "Please allow to write to storage", Toast.LENGTH_LONG).show();
                }
                break;
            }
        }
    }

    private class MyTask extends AsyncTask<RequestBodyGenerateImg2, Integer, String>{
        public ProgressBar progressBar1;
        Response<ResponseGetGeneratedImage> responseGetGeneratedImageResponse = null;
        @Override
        protected String doInBackground(RequestBodyGenerateImg2... requestBodyGenerateImg2) {
            progressBar1.setVisibility(View.VISIBLE);
            RequestBodyGenerateImg2 requestBodyGenerateImg = requestBodyGenerateImg2[0];

            tenunNetworkInterface.generateNewMotif(APIModule.ACCESS_TOKEN_TEMP, requestBodyGenerateImg).enqueue(new Callback<ResponseGetGeneratedImage>() {
                @Override
                public void onResponse(Call<ResponseGetGeneratedImage> call, Response<ResponseGetGeneratedImage> response) {
                    if (response.isSuccessful()) {
                        generatedImg = response.body().getFilename();
                        List<Generate> generate = response.body().getData();
                        updateListGenerate(generate);
                        progressBar1.setVisibility(View.GONE);
                        generatedUrl = response.body().getExec_result();
                    } else {
                        titleResult.setText("Error (" + response.code() + ")");
                        Snackbar snackbar = Snackbar.make(coordinatorLayout, "Error to generate", Snackbar.LENGTH_INDEFINITE);
                        snackbar.setAction("DISMISS", v -> {
                            snackbar.dismiss();
                            titleResult.setText(R.string.generated_text);
                        });
                        snackbar.show();
                        progressBar1.setVisibility(View.GONE);
                        generateMenu.setEnabled(true);
                    }
                }

                @Override
                public void onFailure(Call<ResponseGetGeneratedImage> call, Throwable t) {
                    titleResult.setText(t.getLocalizedMessage());
                    Snackbar snackbar = Snackbar.make(coordinatorLayout, "Error to generate", Snackbar.LENGTH_INDEFINITE);
                    snackbar.setAction("DISMISS", v -> {
                        snackbar.dismiss();
                        titleResult.setText(R.string.generated_text);
                    });
                    snackbar.show();
                    progressBar1.setVisibility(View.GONE);
                    generateMenu.setEnabled(true);
                }
            });

            return "Generate New Motif";
        }

        @Override
        protected void onPreExecute(){
            super.onPreExecute();
            progressBar1.setVisibility(View.VISIBLE);
        }

        @Override
        protected void onPostExecute(String result){
            super.onPostExecute(result);
            progressBar1.setVisibility(View.GONE);
        }
    }
}

